package ch.zhaw.pm2.multichat.client;

import javafx.beans.property.SimpleBooleanProperty;
import javafx.beans.property.SimpleListProperty;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;

import java.util.ArrayList;
import java.util.List;

import static ch.zhaw.pm2.multichat.client.ClientConnectionHandler.USER_NONE;

public class ClientMessageList {
    private List<Message> messages;
    private SimpleBooleanProperty changed;

    public ClientMessageList() {
        messages = new ArrayList<>();
        changed = new SimpleBooleanProperty(true);
    }

    public SimpleBooleanProperty getChangedProperty() {
        return changed;
    }

    public void addMessage(String sender, String receiver, String message, Message.MessageType type) {
        if(sender == null) {
            sender = USER_NONE;
        }
        messages.add(new Message(type, sender, receiver, message));
        changed.set(!changed.get());
    }

	public String getFilteredMessages(String filter) {
        StringBuilder result = new StringBuilder();
		boolean showAll = filter == null || filter.isBlank();
		for(Message message : messages) {
			if(showAll || message.matchesFilter(filter))
			{
                switch (message.getType()) {
                    case MESSAGE -> result.append(String.format("[%s -> %s] %s\n", message.getSender(), message.getReceiver(), message.getText()));
                    case ERROR -> result.append(String.format("[ERROR] %s\n", message.getText()));
                    case INFO -> result.append(String.format("[INFO] %s\n", message.getText()));
                    default -> result.append(String.format("[ERROR] %s\n", "Unexpected message type: " + message.getType()));
                }
			}
		}
        return result.toString();
	}

	public String getMessages() {
        StringBuilder result = new StringBuilder();
        for(Message message : messages) {
            switch (message.getType()) {
                case MESSAGE -> result.append(String.format("[%s -> %s] %s\n", message.getSender(), message.getReceiver(), message.getText()));
                case ERROR -> result.append(String.format("[ERROR] %s\n", message.getText()));
                case INFO -> result.append(String.format("[INFO] %s\n", message.getText()));
                default -> result.append(String.format("[ERROR] %s\n", "Unexpected message type: " + message.getType()));
            }
        }
        return result.toString();
    }

    public void clear() {
        messages = new SimpleListProperty<>();
    }

}
